/*****************************************************************************
**
** parse_cl.h
**
** Sat Mar  7 11:57:10 1998
** mike@cathmike (Mike Borella)
**
** Header file for command line parsing routines
**
** Automatically created by genparse v0.2.2
**
** See http://www.xnet.com/~cathmike/MSB/Software/ for details
**
*****************************************************************************/

#include <stdio.h>

struct arg_t
{
  int c;
  char *i;
  int p;
  int l;
  int n;
  int t;
  int optind;
};

struct arg_t *parse_cl(int, char **);
void usage(char *);
void free_args(struct arg_t *);
