/**************************************************************************** 
** File: ethernet.h
**
** Author: Mike Borella
**
** Comments: Generic ethernet structure - an attempt at OS independence
**
** $Log: ethernet.h,v $
** Revision 1.2  1998/06/12 20:49:07  mborella
** Added log tag
**
**
*****************************************************************************/

#define ETHERTYPE_IP            0x0800  /* IP protocol */
#define ETHERTYPE_ARP           0x0806  /* Addr. resolution protocol */
#define ETHERTYPE_REVARP        0x8035  /* reverse Addr. resolution protocol */
#define ETHERTYPE_IPX           0x8137  /* IPX family */
#define ETHERMTU                1500

/*
 * Ethernet header
 */

typedef struct _EtherHdr
{
  unsigned char  ether_dst[6];
  unsigned char  ether_src[6];
  unsigned short ether_type;
} EtherHdr;







