/**************************************************************************** 
** File: spx.h
**
** Author: Mike Borella
**
** Comments: Dump SPX header format and such
**
** $Log: spx.h,v $
** Revision 1.2  1998/06/12 20:48:13  mborella
** Added log tag
**
**
*****************************************************************************/

typedef struct _SPXhdr
{
  u_int8_t  cc;      /* connection control */
  u_int8_t  ds_type; /* data stream type */
  u_int16_t s_id;    /* source connection id */
  u_int16_t d_id;    /* destination connection id */
  u_int16_t seqno;   /* sequence number */
  u_int16_t ackno;   /* acknowledgement number */
  u_int16_t allocno; /* allocation number */
} SPXhdr;

