/**************************************************************************** 
** File: udp.h
**
** Author: Mike Borella
**
** Comments: Generic UDP header structure - an attempt at OS independence
**
*****************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define L2TP_PORT 1701
#define DHCP_CLIENT_PORT 68
#define DHCP_SERVER_PORT 67

typedef struct _UDPHdr
{
  u_int16_t uh_src;
  u_int16_t uh_dst;
  u_int16_t uh_len;
  u_int16_t uh_chk;
} UDPHdr;
